import React from "react";
import PropTypes from "prop-types";
import Reflux from "reflux";
import {Button, Glyphicon, Panel, Table} from "react-bootstrap";
import JsonLdUtils from "jsonld-utils";

import Actions from "../../../actions/Actions";
import Constants from "../../../constants/Constants";
import EditedEvent from "./EditedEvent";
import I18nWrapper from "../../../i18n/I18nWrapper";
import injectIntl from "../../../utils/injectIntl";
import OptionsStore from "../../../stores/OptionsStore";
import ReportFactory from "../../../model/ReportFactory";
import Utils from "../../../utils/Utils";

class Events extends Reflux.Component {
    constructor(props) {
        super(props);
        this.i18n = props.i18n;
        this.state = {currentEvent: null};
        this.store = OptionsStore;
        this.storeKeys = [Constants.OPTIONS.EVENT_TYPE];
    }

    componentDidMount() {
        Actions.loadOptions(Constants.OPTIONS.EVENT_TYPE);
    }

    _onAdd = () => {
        this.setState({
            currentEvent: ReportFactory.createEvent()
        });
    };

    _onEdit = (event) => {
        this.setState({currentEvent: event});
    };

    _onRemove = (event) => {
        const events = this.props.occurrence.parts;
        events.splice(events.indexOf(event), 1);
        this.props.onChange({parts: events});
    };

    _onSaveEdit = (event) => {
        delete event.isNew;
        let events = this.props.occurrence.parts;
        events = events ? events.slice() : [];
        const index = events.indexOf(this.state.currentEvent);
        if (index !== -1) {
            events.splice(index, 1, event);
        } else {
            events.push(event);
        }
        this.props.onChange({parts: events});
        this._onCloseEdit();
    };

    _onCloseEdit = () => {
        this.setState({currentEvent: null});
    };

    render() {
        return <Panel header={<h3>{this.i18n('occurrence.events')}</h3>} bsStyle='info'>
            <div className='row form-group'>
                <div className='col-xs-1'>
                    <Button bsSize='small' bsStyle='primary' onClick={this._onAdd}>
                        <Glyphicon glyph='plus' className='glyphicon-button'/>
                        {this.i18n('add')}
                    </Button>
                </div>
            </div>
            <Table striped bordered condensed hover>
                <thead>
                <tr>
                    <th className='content-center col-xs-10'>{this.i18n('eventtype.title')}</th>
                    <th className='content-center col-xs-2'>{this.i18n('table-actions')}</th>
                </tr>
                </thead>
                <tbody>
                {this._renderEvents()}
                </tbody>
            </Table>
        </Panel>;
    }

    _renderEvents() {
        const events = this.props.occurrence.parts ? this.props.occurrence.parts : [];
        const options = JsonLdUtils.processTypeaheadOptions(this.state[Constants.OPTIONS.EVENT_TYPE], this.props.intl),
            rows = [];
        for (let i = 0, len = events.length; i < len; i++) {
            const event = events[i];
            if (event === this.state.currentEvent) {
                rows.push(<EditedEvent key={'edited'} event={event} eventTypes={options}
                                       onSave={this._onSaveEdit} onCancel={this._onCloseEdit}/>);
            } else {
                rows.push(<EventRow key={event.id ? event.id : 'new-' + i} event={event} onEdit={this._onEdit}
                                    onRemove={this._onRemove} typeLabel={Utils.idToName(event.type, options)}/>);
            }
        }
        if (this.state.currentEvent && this.state.currentEvent.isNew) {
            rows.push(<EditedEvent key={'edited'} event={this.state.currentEvent} eventTypes={options}
                                   onSave={this._onSaveEdit} onCancel={this._onCloseEdit}/>);
        }
        return rows;
    }
}

Events.propTypes = {
    occurrence: PropTypes.object.isRequired,
    onChange: PropTypes.func.isRequired
};

let EventRow = (props) => {
    const i18n = props.i18n,
        event = props.event;
    return <tr>
        <td className='vertical-middle'>
            {event.type ?
                <a href={event.type} target='_blank' title={event.type}>{props.typeLabel}</a> : props.typeLabel}
        </td>
        <td className='actions'>
            <Button bsStyle='primary' bsSize='small'
                    onClick={() => props.onEdit(event)}>{i18n('table-edit')}</Button>
            <Button bsStyle='warning' bsSize='small'
                    onClick={() => props.onRemove(event)}>{i18n('delete')}</Button>
        </td>
    </tr>;
};

EventRow.propTypes = {
    event: PropTypes.object.isRequired,
    typeLabel: PropTypes.string.isRequired,
    onEdit: PropTypes.func.isRequired,
    onRemove: PropTypes.func.isRequired
};

EventRow = injectIntl(I18nWrapper(EventRow));

export default injectIntl(I18nWrapper(Events));
